-module(notifier_tests).
-import(notifier,[start/1]).
-import(testing,[dotests/2,eqTest/3]).
-export([main/0]).
main() -> compile:file(notifier), dotests("notifier_tests $Revision: 1.3 $", tests()).
tests() -> N1 = start(e1), N2 = start(e2),
    W1 = makeWorker(N1), W2 = makeWorker(N1), W3 = makeWorker(N2),
    lists:foreach(fun(W) -> receive {W, registered} -> ok end end, [W1,W2,W3]),
    [eqTest(observers(N1),"==",[W2,W1]), eqTest(observers(N2),"==",[W3]),
     eqTest(getCount(W1),"==",0), eqTest(getCount(W2),"==",0),
     eqTest(getCount(W3),"==",0),
     eqTest(begin announce(N1), getCount(W1) end,"==",1), eqTest(getCount(W2),"==",1),
     eqTest(begin announce(N1), getCount(W1) end,"==",2), 
     eqTest(getCount(W2),"==",2), eqTest(getCount(W3),"==",0),
     eqTest(begin announce(N2), getCount(W3) end,"==",1), eqTest(getCount(W2),"==",2),
     eqTest(begin announce(N1), getCount(W1) end,"==",3),
     eqTest(getCount(W2),"==",3)                           ].
% functions to help with testing, NOT for you to implement
-spec observers(NS::pid()) -> [pid()].
observers(NS) -> NS ! {self(), observers},
		 receive {NS, LOP} -> LOP end.
-spec announce(NS::pid()) -> ok.
announce(NS) -> NS ! {self(), announce},
		receive {NS, announced} -> ok end.
% A worker helps with testing
-spec makeWorker(NSPid::pid()) -> pid().
makeWorker(NSPid) -> TPid = self(),
		     spawn(fun() -> workerInit(TPid, NSPid) end).
workerInit(TPid,NSPid) -> NSPid ! {self(), register},
			  receive {NSPid, registered} -> TPid ! {self(), registered} end,
			  worker(TPid,NSPid,0).
worker(TPid,NSPid,Count) ->
    receive {TPid, count} -> TPid ! {self(), Count}, 
			     worker(TPid,NSPid,Count);
	    {NSPid, {event, _EN}} -> worker(TPid, NSPid, Count+1)
    end.
getCount(WPid) -> WPid ! {self(), count},
		  receive {WPid, Count} -> Count end.
